unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Forms,
  StdCtrls, ExtCtrls, ComCtrls, Controls, SimpleTCP, CheckLst;

type
  TForm1 = class(TForm)
    Server: TSimpleTCPServer;
    Client1: TSimpleTCPClient;
    Panel1: TPanel;
    GroupBox1: TGroupBox;
    Panel2: TPanel;
    ServerPortLab: TLabel;
    ListenBox: TCheckBox;
    GroupBox4: TGroupBox;
    GroupBox2: TGroupBox;
    Panel4: TPanel;
    Client1HostLab: TLabel;
    Client1PortLab: TLabel;
    Client1HostEdit: TEdit;
    Client1PortEdit: TEdit;
    Connected1Box: TCheckBox;
    GroupBox5: TGroupBox;
    GroupBox3: TGroupBox;
    Client2HostLab: TLabel;
    Client2PortLab: TLabel;
    Client2HostEdit: TEdit;
    Client2PortEdit: TEdit;
    Connected2Box: TCheckBox;
    GroupBox6: TGroupBox;
    Client2: TSimpleTCPClient;
    ConnectionsLab: TLabel;
    ServerOutput: TMemo;
    Client1Output: TMemo;
    Client2Output: TMemo;
    ServerInput: TEdit;
    Client1Input: TEdit;
    Client2Input: TEdit;
    ServerPortEdit: TEdit;
    StatusBar1: TStatusBar;
    GroupBox7: TGroupBox;
    LogBox: TListBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    SendToBox: TCheckListBox;
    procedure ListenBoxClick(Sender: TObject);
    procedure Connected1BoxClick(Sender: TObject);
    procedure Connected2BoxClick(Sender: TObject);
    procedure ServerError(Sender: TObject; Socket, ErrorCode: Integer;
      ErrorMsg: String);
    procedure ServerAccept(Sender: TObject; Client: TSimpleTCPClient;
      var Accept: Boolean);
    procedure ServerClientConnected(Sender: TObject;
      Client: TSimpleTCPClient);
    procedure ServerClientDisconnected(Sender: TObject;
      Client: TSimpleTCPClient);
    procedure Client1Connected(Sender: TObject);
    procedure Client1Disconnected(Sender: TObject);
    procedure ServerInputKeyPress(Sender: TObject; var Key: Char);
    procedure Client1InputKeyPress(Sender: TObject; var Key: Char);
    procedure Client2InputKeyPress(Sender: TObject; var Key: Char);
    procedure Client1Read(Sender: TObject; Buffer: PChar;
      BufLength: Integer);
    procedure Client2Read(Sender: TObject; Buffer: PChar;
      BufLength: Integer);
    procedure ServerClientRead(Sender: TObject; Client: TSimpleTCPClient;
      Buffer: PChar; Length: Integer);
    procedure FormShow(Sender: TObject);
  private
    procedure Log(Sender: TObject; St: String);
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormShow(Sender: TObject);
begin
  Client1HostEdit.Text := Server.LocalIP;
end;

procedure TForm1.Log(Sender: TObject; St: String);
var
  i: Integer;
  Name: String;
  tmpClient: TSimpleTCPClient;
begin
  Name := Sender.ClassName;
  if TComponent(Sender).Tag <> 0 then Name := Name + ' [' + IntTOStr(TComponent(Sender).Tag) + ']';

  LogBox.Items.Add(Name + ' - ' + St);

  LogBox.ItemIndex := LogBox.Items.Count - 1;

  ConnectionsLab.Caption := 'Connections: ' + IntToStr(Server.Connections.Count);
  SendToBox.Clear;
  i := Server.Connections.Count;
  if i <> 0 then
   for i := 0 to i - 1 do
    begin
     tmpClient := Server.Connections[i];
     SendToBox.Items.Add(tmpClient.Host + ':' + IntToStr(tmpClient.Port));
     SendToBox.Checked[i] := True;
    end;

  Connected1Box.Checked := Client1.Connected;
  Connected2Box.Checked := Client2.Connected;

  ServerPortLab.Enabled := not Server.Listen;
  ServerPortEdit.Enabled := not Server.Listen;
  Client1HostLab.Enabled := not Client1.Connected;
  Client2HostLab.Enabled := not Client2.Connected;
  Client1HostEdit.Enabled := not Client1.Connected;
  Client2HostEdit.Enabled := not Client2.Connected;
  Client1PortLab.Enabled := not Client1.Connected;
  Client2PortLab.Enabled := not Client2.Connected;
  Client1PortEdit.Enabled := not Client1.Connected;
  Client2PortEdit.Enabled := not Client2.Connected;

  ServerInput.Enabled := Server.Listen;
  Client1Input.Enabled := Client1.Connected;
  Client2Input.Enabled := Client2.Connected;
end;

procedure TForm1.ListenBoxClick(Sender: TObject);
begin
  try
    Server.Port := StrToInt(ServerPortEdit.Text);
  except
  end;
  Server.Listen := ListenBox.Checked;

  if Server.Listen then
   Log(Server, 'Server ready')
  else
   Log(Server, 'Server shutdown'); 
end;

procedure TForm1.Connected1BoxClick(Sender: TObject);
begin
  try
    Client1.Host := Client1HostEdit.Text;
    Client1.Port := StrToInt(Client1PortEdit.Text);
  except
  end;
  Client1.Connected := Connected1Box.Checked;
end;

procedure TForm1.Connected2BoxClick(Sender: TObject);
begin
  try
    Client2.Host := Client2HostEdit.Text;  
    Client2.Port := StrToInt(Client2PortEdit.Text);
  except
  end;
  Client2.Connected := Connected2Box.Checked;
end;

procedure TForm1.ServerInputKeyPress(Sender: TObject; var Key: Char);
var
  St: String;
  i: Integer;
  tmpClient: TSimpleTCPClient;
begin
  if Key = #13 then
   begin
    St := 'Server: ' + ServerInput.Text;

    // Sending to checked clients
    // --
    // btw, if you'd like to send msg to all clients
    // you can use "SendToAll" method
    // --
    // Server.SendToAll(PChar(St), Length(St));

    i := Server.Connections.Count;
    if i <> 0 then
     for i := 0 to i - 1 do
      begin
       tmpClient := Server.Connections[i];
       if SendToBox.Checked[i] then
        Server.Send(tmpClient, PChar(St), Length(St));
      end;

    ServerInput.Text := '';
    Key := #0;
   end;
end;

procedure TForm1.Client1InputKeyPress(Sender: TObject; var Key: Char);
var
  St: String;
begin
  if Key = #13 then
   begin
    St := 'Client1: ' + Client1Input.Text;
    Client1.Send(PChar(St), Length(St));

    Client1Input.Text := '';
    Key := #0;
   end;
end;

procedure TForm1.Client2InputKeyPress(Sender: TObject; var Key: Char);
var
  St: String;
begin
  if Key = #13 then
   begin
    St := 'Client2: ' + Client2Input.Text;
    Client2.Send(PChar(St), Length(St));

    Client2Input.Text := '';
    Key := #0;    
   end;
end;

procedure TForm1.ServerError(Sender: TObject; Socket, ErrorCode: Integer;
  ErrorMsg: String);
begin
  Log(Sender, 'ERROR: ' + ErrorMsg);
end;

procedure TForm1.ServerAccept(Sender: TObject; Client: TSimpleTCPClient;
  var Accept: Boolean);
begin
  Log(Sender, 'Client Accepted:  (' + Client.Host + ')');
end;

procedure TForm1.ServerClientConnected(Sender: TObject;
  Client: TSimpleTCPClient);
begin
  Log(Sender, 'Client  (' + Client.Host + ') connected to server');
end;

procedure TForm1.ServerClientDisconnected(Sender: TObject;
  Client: TSimpleTCPClient);
begin
  Log(Sender, 'Client (' + Client.Host + ') disconnected from server.');
end;

procedure TForm1.Client1Connected(Sender: TObject);
begin
  Log(Sender, 'Client connected.');
end;

procedure TForm1.Client1Disconnected(Sender: TObject);
begin
  Log(Sender, 'Client disconnected.');
end;

procedure TForm1.ServerClientRead(Sender: TObject;
  Client: TSimpleTCPClient; Buffer: PChar; Length: Integer);
begin
  Log(Sender, 'Data received from socket ' + IntToStr(Client.Socket) + ' (' + Client.Host + ')');
  ServerOutput.Lines.Add(StrPas(Buffer));
end;

procedure TForm1.Client1Read(Sender: TObject; Buffer: PChar;
  BufLength: Integer);
begin
  Log(Sender, 'Data received from socket');
  Client1Output.Lines.Add(StrPas(Buffer));
end;

procedure TForm1.Client2Read(Sender: TObject; Buffer: PChar;
  BufLength: Integer);
begin
  Log(Sender, 'Data received from server');
  Client2Output.Lines.Add(StrPas(Buffer));
end;

end.
